/*
 * LIBPKI - Easy PKI Library
 * by Massimiliano Pala (madwolf@openca.org)
 * OpenCA project 2007
 *
 * Copyright (c) 2007 The OpenCA Project.  All rights reserved.
 *
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

#include <libpki/pki.h>

#ifdef HAVE_LDAP


LDAP *URL_LDAP_connect( URL *url ) {

	LDAP		*ld = NULL;
	int		protocol = -1;
	int		ret = 0;
	int32_t		tout = 10;

	char		*ldap_server = NULL;

#if (LIBPKI_OS_CLASS == LIBPKI_OS_POSIX)
	(void) signal( SIGPIPE, SIG_IGN );
#endif

	if( (!url) || (!url->addr) || (!url->path)) {
		return NULL;
	}

#if defined(LDAP_VENDOR_OPENLDAP)
	if((ldap_server = PKI_Malloc ( strlen(url->addr) + 20 )) == NULL ) {
		return NULL;
	}

	snprintf( ldap_server, strlen(url->addr) + 19, 
			"ldap://%s:%d", url->addr, url->port );

    if((ldap_initialize( &ld, ldap_server )) != LDAP_SUCCESS ) {
#else
	ldap_server = strdup( url->addr );
	if((ld = ldap_init( ldap_server, url->port)) == NULL ) {
#endif
		PKI_Free ( ldap_server );

		PKI_log_err ( "ERROR::Can not initialize LDAP connection to %s",
			url->addr );
		return ( NULL );
	}

	PKI_Free ( ldap_server );

	(void) ldap_set_option( ld, LDAP_OPT_PROTOCOL_VERSION, &protocol );

#if defined(LDAP_VENDOR_OPENLDAP)
    (void) ldap_set_option( ld, LDAP_OPT_TIMELIMIT, (unsigned long *) &tout );
#endif

#if defined(LDAP_VENDOR_MICROSOFT)
    (void) ldap_set_option( ld, LDAP_OPT_TCP_KEEPALIVE, LDAP_OPT_ON );
#endif

#if defined(LDAP_VENDOR_MICROSOFT) || defined(LDAP_VENDOR_SUN)
    if( ldap_bind ( ld, url->usr, url->pwd, LDAP_AUTH_SIMPLE) != 
				LDAP_SUCCESS ) {
		PKI_log_err("LDAP::Can not bind to %s", url->addr );
        return NULL;
    };
#elif defined(LIBPKI_MACOS)
    // Not Supported
#elif defined(LDAP_VENDOR_OPENLDAP)
#  ifdef LDAP_OPT_X_KEEPALIVE_IDLE
    (void) ldap_set_option( ld, LDAP_OPT_X_KEEPALIVE_IDLE, LDAP_OPT_ON );
#  endif
# endif

	PKI_log_debug("LDAP::Initialization Successful!");
	return(ld);
}

PKI_MEM_STACK *URL_get_data_ldap_url( URL *url, ssize_t size ) {

	LDAP	*ld = NULL;
	int  	i,rc;
	char	*attrs[] = { url->attrs, NULL } ;
	char    *filter = "objectclass=*";
	struct berval **vals = NULL;
	LDAPMessage *res = NULL;
	PKI_MEM_STACK *ret = NULL;
	PKI_MEM *obj = NULL;

	if( (!url) || (!url->addr) || (!url->path)) {
		return NULL;
	}

	if((ld = URL_LDAP_connect ( url )) == NULL ) {
		return NULL;
	};

	/* We search for the exact match, so LDAP_SCOPE_BASE is used here */
	if (( rc = ldap_search_s( ld, url->path, LDAP_SCOPE_BASE,
			filter, attrs, 0, &res )) != LDAP_SUCCESS ) {

		if ( rc != LDAP_NO_SUCH_OBJECT ) {
			PKI_log_err("LDAP: [%s] object not found", url->path);
		}

		goto end;
	}

	i = ldap_count_entries( ld, res );

	if((ret = PKI_STACK_MEM_new()) == NULL ) {
		/* ERROR: Allocating memory */
		goto end;
	}
	if((vals = ldap_get_values_len (ld, res, attrs[0])) != NULL ) {

		for( i=0; vals[i] != NULL; i++ ) {
			if((obj = PKI_MEM_new_null()) == NULL ) {
				goto end;
			}
			if(PKI_MEM_add( obj, (char *) vals[i]->bv_val, 
					vals[i]->bv_len) == PKI_ERR) {
				/* ERROR in memory growth */;
				break;
			}
			PKI_STACK_MEM_push( ret, obj );
		}

		ldap_value_free_len( vals );
		vals = NULL;
	} else {
		return NULL;
	}

end:
	if(res) ldap_msgfree( res );
	if(ld) ldap_unbind( ld );

	return (ret);
}

#endif /* LDAP */
